import asyncio
import logging
from contextlib import AsyncExitStack
from types import TracebackType
from typing import Self
from collections import Counter

from uptime.frigate_config import FrigateConfig
from uptime.notify import NtfyNotifier
from uptime.ping import ip_ping_all

logger = logging.getLogger(__name__)


class CameraMonitor:
    def __init__(self, wait_time: int = 60, consecutive_down_threshold: int = 3):
        self._wait_time = wait_time
        self._consecutive_down_threshold = consecutive_down_threshold
        self._camera_downtime = Counter()
        self._frigate_config = FrigateConfig()
        self._ntfy_notifier = NtfyNotifier

    async def _on_camera_up(self, camera: str) -> None:
        logger.info(f"Camera {camera} is back online")
        await self._ntfy_notifier.send_notification(camera, True)

    async def _on_camera_down(self, camera: str) -> None:
        await self._frigate_config.refresh()
        if camera not in self._frigate_config.active_cameras:
            return

        logger.info(f"Camera {camera} is down")
        await self._ntfy_notifier.send_notification(camera, False)

    async def _on_camera_ping(self, camera: str, success: bool) -> None:
        if success:
            if self._camera_downtime[camera] >= self._consecutive_down_threshold:
                await self._on_camera_up(camera)
            self._camera_downtime[camera] = 0
        else:
            if self._camera_downtime[camera] == self._consecutive_down_threshold:
                await self._on_camera_down(camera)
            self._camera_downtime[camera] += 1

    async def run(self) -> None:
        await self._frigate_config.refresh()
        while True:
            camera_ips = {camera: address.split(":", 1)[0] for camera, address in self._frigate_config.active_camera_addresses.items()}
            ping_results = await ip_ping_all(*camera_ips.values())
            for i, camera in enumerate(camera_ips):
                await self._on_camera_ping(camera, ping_results[i])

            logger.debug(f"Sleeping for {self._wait_time} seconds...")
            await asyncio.sleep(self._wait_time)

